@@ -5,12 +5,17 @@ from collections import defaultdict, OrderedDict
 from datetime import date
 import textwrap
 import gzip
-from Constants import *
+import os
+import sys
 
 try:
     from ORForise.utils import sortORFs  # Calling from ORForise via pip
+    from .Constants import *
 except (ModuleNotFoundError, ImportError, NameError, TypeError) as error:
-    from ORForise.src.ORForise.utils import sortORFs  # Calling from ORForise locally (StORF_Reporter and ORForise in same dir)
+    sys.path.insert(0, '../../../ORForise/src/ORForise/') # Calling from ORForise locally (StORF_Reporter and ORForise in same dir)
+    from utils import sortORFs
+    from Constants import *
+
 
 
 
@@ -165,8 +170,12 @@ def prepare_out(options, storfs, seq_id):
         pos_ = pos.split(',')
         start_stop = sequence[0:3]
         if len(pos_) == 3:
-            mid = int(pos_[1]) - int(pos_[0])
-            mid_stop = sequence[mid-3:mid]
+            if strand == '-':
+                mid = int(pos_[2]) - int(pos_[1])
+                mid_stop = sequence[mid:mid + 3]
+            else:
+                mid = int(pos_[1]) - int(pos_[0])
+                mid_stop = sequence[mid:mid + 3]
         else:
             mid_stop = 'N/A'
         end_stop = sequence[-3:]
@@ -282,7 +291,7 @@ def find_storfs(working_frame,sequence_id,stops,sequence,storfs,short_storfs,con
                         if not first and stop == prev_next_stop:
                             if prev_next_stop != con_StORF_tracker:
                                 con_StORF_tracker = next_stop
-                                seq = sequence[prev_stop:next_stop]
+                                seq = sequence[prev_stop:next_stop + 3]
                                 length = next_stop - prev_stop
                                 ##### Needed to correct for negative frame loci
                                 if working_frame == 'negative':
@@ -298,7 +307,8 @@ def find_storfs(working_frame,sequence_id,stops,sequence,storfs,short_storfs,con
                                 con_StORF_tracker = next_stop
                                 prev_con_StORF = next(reversed(con_StORFs.keys())) #Get last key
                                 seq_start = int(prev_con_StORF.split(',')[0])
-                                seq = sequence[seq_start:next_stop  + 3]
+                                #seq = sequence[seq_start:next_stop  + 3]
+                                seq = sequence[int(prev_con_StORF.split(',')[0]):next_stop + 3]
                                 length = next_stop - seq_start # Check
                                 prev_con_StORF = prev_con_StORF_CHECKER(prev_con_StORF,sequence,options)
                                 ##### Needed to correct for negative frame loci
@@ -343,7 +353,7 @@ def find_storfs(working_frame,sequence_id,stops,sequence,storfs,short_storfs,con
                                                                                                StORF_idx]})
                                 #####
                                 StORF_idx +=1
-                                seen_stops.append(next_stop  + 3)
+                                seen_stops.append(next_stop)#  + 3)
                                 prev_storf = storf
                                 prev_stop = stop
                                 prev_next_stop = next_stop # Check
@@ -635,13 +645,11 @@ def StORF_Reported(options, Contigs):
 
 
 def main():
-    print("Thank you for using StORF-Reporter\nPlease report any issues to: https://github.com/NickJD/StORF-Reporter/issues\n#####")
-
     parser = argparse.ArgumentParser(description='StORF-Reporter ' + StORF_Reporter_Version + ': StORF-Finder Run Parameters.')
     parser._action_groups.pop()
 
     required = parser.add_argument_group('Required Arguments')
-    required.add_argument('-f', action="store", dest='fasta', required=True,
+    required.add_argument('-f', action="store", dest='fasta', required=False,
                         help='Input FASTA File - (UR_Extractor output)')
 
     optional = parser.add_argument_group('Optional Arguments')
@@ -671,13 +679,13 @@ def main():
                         help='Default - True. Only report Short-StORFs?')
     optional.add_argument('-stop_ident', action="store", dest='stop_ident', default=False, choices=[True, False],
                         help='Default - True: Identify Stop Codon positions with \'*\'')
-    optional.add_argument('-type', action='store', dest='feature_type', default='StORF', const='StORF', nargs='?',
+    optional.add_argument('-f_type', action='store', dest='feature_type', default='CDS', const='StORF', nargs='?',
                         choices=['StORF', 'CDS', 'ORF'],
                         help='Default - "StORF": Which GFF feature type for StORFs to be reported as in GFF')
-    optional.add_argument('-minorf', action="store", dest='min_orf', default=100, type=int,
-                        help='Default - 100: Minimum StORF size in nt')
-    optional.add_argument('-maxorf', action="store", dest='max_orf', default=50000, type=int,
-                        help='Default - 50kb: Maximum StORF size in nt')
+    optional.add_argument('-minorf', action="store", dest='min_orf', default=99, type=int,
+                        help='Default - 99: Minimum StORF size in nt')
+    optional.add_argument('-maxorf', action="store", dest='max_orf', default=60000, type=int,
+                        help='Default - 60kb: Maximum StORF size in nt')
     optional.add_argument('-codons', action="store", dest='stop_codons', default="TAG,TGA,TAA",
                         help='Default - (\'TAG,TGA,TAA\'): List Stop Codons to use')
     optional.add_argument('-olap', action="store", dest='overlap_nt', default=50, type=int,
@@ -689,13 +697,16 @@ def main():
                         help='Default - Start Position: How should StORFs be ordered when >1 reported in a single UR.')
     optional.add_argument('-spos', action="store", dest='stop_inclusive', default=True, type=eval, choices=[True, False],
                         help='Default - False: Print out StORF positions inclusive of first stop codon')
-    optional.add_argument('-o', action="store", dest='out_file', required=False,
-                        help='Default - False: Without filetype - default appends \'_StORF-R\' to end of input gff filename (replaces \'.gff\')')
-    optional.add_argument('-af', action="store", dest='affix', required=False,
-                        help='Default - None: \'-af Con-StORFs\' can be used to append an identifier to output filename '
-                             'to distinguish Con-StORF from StORF runs)')
+
 
     output = parser.add_argument_group('Output')
+    output.add_argument('-oname', action="store", dest='o_name', required=False,
+                        help='Default - Appends \'_StORF-R\' to end of input FASTA filename')
+    output.add_argument('-odir', action="store", dest='o_dir', required=False,
+                        help='Default -  Same directory as input FASTA')
+    # output.add_argument('-af', action="store", dest='affix', required=False,
+    #                     help='Default - \'StORF-R\': can be used to append an identifier to output filename '
+    #                          'to distinguish Con-StORF from StORF runs)')
     output.add_argument('-gff', action='store', dest='gff', default=True, type=eval, choices=[True, False],
                         help='Default - True: Output a GFF file')
     output.add_argument('-aa', action="store", dest='translate', default=False, type=eval, choices=[True, False],
@@ -705,17 +716,30 @@ def main():
     output.add_argument('-lw', action="store", dest='line_wrap', default=True, type=eval, choices=[True, False],
                         help='Default - True: Line wrap FASTA sequence output at 60 chars')
     output.add_argument('-gff_fasta', action="store", dest='gff_fasta', default=False, type=eval, choices=[True, False],
-                        help='Default - False: Report all gene sequences (nt) at the bottom of GFF files in PROKKA output mode')
+                        help='Default - False: Report all gene sequences (nt) at the bottom of GFF files in Prokka output mode')
     output.add_argument('-gz', action='store', dest='gz', default='False', type=eval, choices=[True, False],
                         help='Default - False: Output as .gz')
     optional.add_argument('-nout', action='store', dest='nout', default='False', type=eval, choices=[True, False],
                         help=argparse.SUPPRESS)
 
     misc = parser.add_argument_group('Misc')
-    misc.add_argument('-v', action='store', dest='verbose', default='False', type=eval, choices=[True, False],
-                        help='Default - False: Print out runtime status')
+    misc.add_argument('-verbose', action='store', dest='verbose', default=False, type=eval, choices=[True, False],
+                        help='Default - False: Print out runtime messages')
+    misc.add_argument('-v', action='store_true', dest='version',
+                        help='Default - False: Print out version number and exit')
 
     options = parser.parse_args()
+
+    if options.fasta == None:
+        if options.version:
+            sys.exit(StORF_Reporter_Version)
+        else:
+            exit('StORF-Finder: error: the following arguments are required: -f')
+
+    print("Thank you for using StORF-Reporter -- A detailed user manual can be found at https://github.com/NickJD/StORF-Reporter\n"
+          "Please report any issues to: https://github.com/NickJD/StORF-Reporter/issues\n#####")
+
+
     #ns_nt = defaultdict  # Used to Record non-standard nucleotides - not implemented yet
     ##### Load in fasta file
     sequences = OrderedDict()
@@ -729,47 +753,53 @@ def main():
             sequence_regions, sequences =  fasta_load(fasta_in, sequence_regions, sequences)
         if options.verbose == True:
             print(fasta_in.name)
-    ####
-    if options.out_file == None: # Clunky
-        tmp_outfile = options.fasta.split('.')[-1]
-        options.out_file = options.fasta.replace(tmp_outfile,'')
-        options.out_file = options.out_file[:-1]
-
-    if options.affix == None: # Also Clunky
-        affix = '_StORF-R'
-    else:
-        affix = '_StORF-R_' + options.affix
+    #### Output Directory and Filename handling
+    if options.o_dir == None and options.o_name == None:
+        tmp_extension = options.gff.split('.')[-1]  # could be .fa/.fasta etc
+        output_file = options.gff.replace('.' + tmp_extension, '')
+        output_file = output_file + '_StORF-R'
+    elif options.o_dir != None and options.o_name != None:
+        output_file = options.o_dir
+        output_file = output_file + options.o_name
+    elif options.o_dir != None:
+        tmp_extension = options.gff.split('.')[-1]  # could be .fa/.fasta etc
+        output_file = options.gff.replace('.' + tmp_extension, '').split('/')[-1]
+        output_file = options.o_dir + output_file + '_StORF-R'
+    elif options.o_name != None:
+        tmp_filename = options.gff.split('/')[-1]  # could be .fa/.fasta etc
+        output_file = options.gff.replace(tmp_filename, '')
+        output_file = output_file + options.o_name
 
     if not options.gz: # Clear fasta and gff files if not empty - Needs an elegant solution
         if not options.aa_only:
-            gff_out = open(options.out_file + affix + '.gff', 'w', newline='\n', encoding='utf-8')
+            gff_out = open(output_file + '.gff', 'w', newline='\n', encoding='utf-8')
             gff_out.write("##gff-version\t3\n#\tStORF-Reporter - Stop ORF Predictions\n#\tRun Date:" + str(date.today()) + '\n')
             gff_out.write('##StORF-Reporter ' + StORF_Reporter_Version + '\n')
             for seq_reg in sequence_regions:
                 gff_out.write(seq_reg + '\n')
-            gff_out.write("##Original File: " + options.fasta + '\n\n')
-            fasta_out = open(options.out_file + affix + '.fasta', 'w', newline='\n', encoding='utf-8')
+            gff_out.write("##Original File: " + options.gff.split('/')[-1] + '\n\n')
+            fasta_out = open(output_file + '.fasta', 'w', newline='\n', encoding='utf-8')
             if options.translate:
-                aa_fasta_out = open(options.out_file + affix + '_aa.fasta', 'w', newline='\n', encoding='utf-8')
+                aa_fasta_out = open(output_file + '_aa.fasta', 'w', newline='\n', encoding='utf-8')
             else:
                 aa_fasta_out = None
         elif options.aa_only:
-            aa_fasta_out = open(options.out_file + affix + '_aa.fasta', 'w', newline='\n', encoding='utf-8')
+            aa_fasta_out = open(output_file + '_aa.fasta', 'w', newline='\n', encoding='utf-8')
     elif options.gz:
         if not options.aa_only:
-            gff_out = gzip.open(options.out_file + affix + '.gff.gz', 'wt', newline='\n', encoding='utf-8')
+            gff_out = gzip.open(output_file + '.gff.gz', 'wt', newline='\n', encoding='utf-8')
             gff_out.write("##gff-version\t3\n#\tStORF-Reporter - Stop ORF Predictions\n#\tRun Date:" + str(date.today()) + '\n')
             gff_out.write('##StORF-Reporter ' + StORF_Reporter_Version + '\n')
             for seq_reg in sequence_regions:
                 gff_out.write(seq_reg + '\n')
-            gff_out.write("##Original File: " + options.fasta + '\n\n')
-            fasta_out = gzip.open(options.out_file + affix + '.fasta.gz', 'wt', newline='\n', encoding='utf-8')
+            gff_out.write("##Original File: " + options.gff + '\n\n')
+            fasta_out = gzip.open(output_file + '.fasta.gz', 'wt', newline='\n', encoding='utf-8')
             if options.translate:
-                aa_fasta_out = gzip.open(options.out_file + affix + '_aa.fasta.gz', 'wt', newline='\n', encoding='utf-8')
+                aa_fasta_out = gzip.open(output_file + '_aa.fasta.gz', 'wt', newline='\n', encoding='utf-8')
             else:
                 aa_fasta_out = None
         elif options.aa_only:
-            aa_fasta_out = gzip.open(options.out_file + affix + '_aa.fasta.gz', 'wt', newline='\n', encoding='utf-8')
+            aa_fasta_out = gzip.open(output_file + '_aa.fasta.gz', 'wt', newline='\n', encoding='utf-8')
 
     for sequence_id, sequence_info in sequences.items():
         if len(sequence_info[1]) >= options.min_orf:
